import pytest
from treecrawl.utility import locate_subdir


def pytest_addoption(parser):
    parser.addoption(
        "--update_golden",
        action="store_true",
        help="Update golden files before running tests",
    )


@pytest.fixture
def update_golden(request):
    return request.config.getoption("--update_golden")


@pytest.fixture(scope="session", autouse=True)
def testdata():
    return locate_subdir("testdata")



import logging
import sys
from typing import List


def create_module_logger(mn):
    ml = logging.getLogger(mn)
    ml.setLevel(logging.INFO)

    ch = logging.StreamHandler(sys.stdout)

    formatter = logging.Formatter(
        "%(asctime)s - {%(name)s} - "
        "{%(filename)s:%(funcName)s:%(lineno)d} - "
        "%(levelname)s - %(message)s"
    )
    ch.setFormatter(formatter)

    ml.addHandler(ch)
    return ml


module_name = str(__name__)
module_logger = create_module_logger(module_name)


def find_path_to_ancestor(target_dir=None):
    """Find path to parent directory among the ancestors

    target_dir is the base name of a directory  (without path separator) that
    will be discovered from amogn the parent directories

    If the target_dir is given a strinf that contains a path separator, it just
    returns the target_dir.  This is useful for testing, so we can explicitly
    pass an absolute path.

    :param str target_dir: target directory

    :rtype: str
    """
    import os

    if os.path.sep in target_dir:
        logging.info(
            "Target directory contains path separators.  "
            "Returning the parameter itself"
        )
        return target_dir
    logging.info("Searching ancestors for directory: {}".format(target_dir))
    ancestors = os.getcwd().split(os.path.sep)
    if target_dir not in ancestors:
        raise RuntimeError(
            "Current directory is not a subdirectory of {}".format(target_dir)
        )
    else:
        # iterate through the last dirs until we hit the one we want
        while not ancestors[-1] == target_dir:
            ancestors.pop()
    # rejoin: we lose the first separator in the split/join
    return os.path.sep + os.path.join(*ancestors)


def find_path_to_subdirectory(target_dir, search_path=None) -> List[str]:
    """Find a directory name among subdirs

    searching for target_dir: 'testdata' from '/home/nate':
    '/home/nate/testdata' would match
    '/home/nate/testdata/something' would NOT match
    '/home/nate/testdata/something/testdata' would  match

    The result would be a list containing both matching paths


    Args:
        target_dir (str): Base directory name to search for among
            sub-directories

        search_path (str, optional): If no search path is provided, start in
        current directory. Defaults to None.

    Returns:
        List[str]: List of absolute paths to subdirectories matching the
        target_dir
    """
    import os

    if search_path is None:
        search_path = os.getcwd()
    res = []  # type: List[str]

    def contains_target(tp) -> bool:
        words = tp.split(os.path.sep)
        return target_dir == words[-1]

    all_subdirs = [x[0] for x in os.walk(search_path)]
    for sd in all_subdirs:
        if contains_target(sd):
            res.append(sd)
    res.sort()
    return res


def file_to_string(file_path):
    """Return file contents as a string

    :param str file_path: absolute path to a file
    :rtype: str

    """
    with open(file_path, "rb") as f:
        try:
            data = f.read()
        except UnicodeDecodeError as err:
            msg = """Failed to read file: {0}
            You probably need to set the locale in your environment
            This is often a proiblem inside docker containers
            something like this might work:
            RUN locale-gen en_US.UTF-8
            ENV LANG en_US.UTF-8
            ENV LANGUAGE en_US:en
            ENV LC_ALL en_US.UTF-8
            """.format(
                file_path
            )
            raise RuntimeError(msg) from err
    return data.decode("utf8", "ignore")


def string_to_file(input_string, file_path):
    """Write/Over-write a file's contents with a string


    :param str input_string: string data
    :param str file_path: absolute path to a file
    """
    with open(file_path, "w") as f:
        f.write(input_string)


def string_to_log_level(log_level_string):
    """Given a string convert it to a logging level for use by logging.log

    Accept the string ('CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG') or the
    integer ('10', '20', '30', '40', '50') format

    :param str log_level_string: case insensitive

    """
    import logging

    if log_level_string.upper() in [
        "CRITICAL",
        "ERROR",
        "WARNING",
        "INFO",
        "DEBUG",
    ]:
        return getattr(logging, log_level_string.upper())

    integer_levels = {
        "50": "CRITICAL",
        "40": "ERROR",
        "30": "WARNING",
        "20": "INFO",
        "10": "DEBUG",
    }
    return getattr(logging, integer_levels[log_level_string])


def mkdir_p(target, is_file=False):
    """Create the directory path to the target.
    If the target is a file, create the path to its parent (directory)

    :param str target: path to a target directory or file
    :param bool is_file: Indicates whether the target is a file


    :rtype: str
    """
    import pathlib
    import errno
    import os

    path = pathlib.Path(target)
    if is_file:
        path = path.parent

    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno != errno.EEXIST:
            raise
        pass
    return path


def get_all_files(target_dir):
    """Recurse the all subdirs and list os abs paths

    :param str target_dir: Indicates whether the target is a file


    :rtype: List[str]
    """
    import os

    res = []
    for root, d_names, f_names in os.walk(target_dir):
        for f in f_names:
            res.append(os.path.join(root, f))
    return res


def strip_suffix(s, suffix):
    """Remove suffix frm the end of s
    is s = "aaa.gpg" and suffix = ".gpg", return "aaa"
    if s is not a string return None
    if suffix is not a string, return s

    :param str s: string to modify
    :param str suffix: suffix to remove

    :rtype: Optional[str]=None
    """
    if not isinstance(s, str):
        return None
    if not isinstance(suffix, str):
        return s
    if s.endswith(suffix):
        return s[: -len(suffix)]
    return s


def strip_prefix(s, prefix):
    """Remove prefix frm the beginning of s
    is s = "some_something" and prefix = "some_", return "something"
    if s is not a string return None
    if prefix is not a string, return s

    :param str s: string to modify
    :param str prefix: prefix to remove

    :rtype: Optional[str]=None
    """
    if not isinstance(s, str):
        return None
    if not isinstance(prefix, str):
        return s
    if s.startswith(prefix):
        return s[len(prefix) :]  # noqa: E203
    return s


def validate_path(ip=None):
    """Give a valid path return absolute

    If the provided relative or absoltue path is invalide, raise
    RuntimeError exception

    If none is provided, use current working directory

    :param str ip: absolute or relative path

    :rtype: str
    """
    import os

    if ip is None:
        ip = os.getcwd()
    rp = os.path.realpath(ip)
    if os.path.exists(rp):
        return rp
    else:
        raise RuntimeError("Invalid path: {}".format(rp))


#
# def path_type(ip):
#     """ return a string describing the ype of object represented by a path
#
#     examples:
#      - "dir" (if it's a directory)
#      - 'text/plain; charset=us-ascii' (mimetype if not)
#
#     :param str ip: absolute path to file or directory
#
#     :rtype: str
#     """
#     import magic
#     import os
#     if os.path.isdir(ip):
#         return "dir"
#
#     if os.path.isfile(ip):
#         m = magic.Magic()
#         return m.from_file(ip)


def file_name_from_path(ff):
    """Return the file name given a path

    "a/b/g/file.txt" -> "file.txt

    :param str ff: pathto file

    :rtype: str
    """
    import os

    words = ff.split(os.path.sep)
    return words[-1]


def output_file_from_input_file(input_root, output_root, input_file):
    # normalize root formats to make sure both have trialing slashes
    import os

    in_r = os.path.join(input_root, "")
    out_r = os.path.join(output_root, "")
    # /a/b/c/file.txt, /a/b/ -> c/file.txt
    file_relative = strip_prefix(input_file, in_r)
    # if there's a leading slash, remove it
    if file_relative.startswith(os.path.sep):
        file_relative = file_relative[1:]
    res = os.path.join(out_r, file_relative)
    return res


def locate_subdir(target_dir):
    """Recursively look for subdir and return absolute path

    This is really useful in locating resource directories  for tests when
    the test could be run from the project directory or tests directory.

    :param str target_dir: directory to locate

    :rtype: List[Dict[str, str]]
    """
    import os

    for root, d_names, f_names in os.walk(os.getcwd()):
        for d in d_names:
            if d == target_dir:
                return os.path.join(root, d)
    return None


def compare_directories(d1, d2):
    """Compare directories that should be exact matches



    :param str d1: left directory
    :param str d2: left directory


    :rtype: bool
    """
    import filecmp

    # iterate d1 files and compare to companion d2 file
    for left in get_all_files(d1):
        right = output_file_from_input_file(d1, d2, left)
        if not filecmp.cmp(left, right, shallow=False):
            return False

    # iterate d2 files and compare to companion d1 file
    for right in get_all_files(d2):
        left = output_file_from_input_file(d2, d1, right)
        if not filecmp.cmp(right, left, shallow=False):
            return False
    return True



#!/usr/bin/env python

"""Tests for `treecrawl` package."""
import os
import pytest
from treecrawl.transformer import Transformer
from treecrawl.casehelper import CaseHelper


class MakeUpper(Transformer):
    """Convert non-ASCII files to ASCII"""

    def __init__(self, input, output, dry_run=False):
        super().__init__(input=input, output=output, dry_run=dry_run)

    def is_target(self, i_file):
        included_extensions = [".txt"]

        # if it's not a file, right?
        if not os.path.isfile(i_file):
            return False

        # Regardless of extension if the file is in a .git directory
        # exclude it
        if ".git" in i_file.split(os.path.sep):
            return False

        # now target only files ending in ".txt
        # i could use
        _, ext = os.path.splitext(i_file)
        if ext in included_extensions:
            return True

        return False

    def transform(self, source_file, destination_file):
        from treecrawl.utility import file_to_string

        contents = file_to_string(source_file)
        contents = contents.upper()
        self.write_string_to_output(contents, destination_file)


@pytest.mark.parametrize(
    "test_case",
    ["pets", "cities"],
)
def test_populate_temp(test_case, tmp_path, request, testdata, update_golden):
    """Run and compare results to expected"""
    import os
    from treecrawl.utility import compare_directories

    # Instantiate a CaseHelper object with the name of the test (originalname)
    c = CaseHelper(
        testdata, request.node.originalname, test_case, str(tmp_path)
    )
    assert compare_directories(c.golden, c.expected)
    assert compare_directories(
        os.path.join(c.temp_case_dir, "input"),
        os.path.join(c.project_case_dir, "input"),
    )


@pytest.mark.parametrize(
    "test_case",
    ["pets", "cities"],
)
def test_make_upper(test_case, tmp_path, request, testdata, update_golden):
    c = CaseHelper(
        testdata,
        request.node.originalname,
        test_case,
        str(tmp_path),
        update_golden=update_golden,
    )

    """when update golden is set by running pytest --update_golden,
    the project golden files are deleted. This step generates new ones from
    the the function under test """
    if update_golden:
        _ = MakeUpper(c.input, c.golden)

    m = MakeUpper(c.input, c.actual)
    m.run()
    for r in c.compare():
        succeeded, compared = r
        assert succeeded
        if not succeeded:
            print("input: {}\nactual: {}\nexpected: {}".format(*compared))



"""Helpers for using test data


"""
import os
from .utility import mkdir_p
from typing import List, Tuple  # noqa


class CaseHelper(object):
    """Test helper object representing the test and test case data

    It should log the relevant temp path locations

    Assumptions:
        - The project test resources are in a directory 'testdata'
        - Functions under test accept input and output (actual) locations

    CaseHelper represents test case and golden data for projects that transform
    files or directory contents

    golden(str): absolute path to a golden resource (file or directory) in the
    project tree. This is updated when pytest is run with the -update-golden
    flag. example: testdata/[TEST]/[CASE]/output.golden

    expected(str): absolute path to a copy of the golden resource in the
    tmp/test/case used for comparison. It can be useful to keep all of the
    relevant data for a test run in one place. example:
    [TMP_DIR]/[TEST]/[CASE]/

    input(str): absolute path to the transformation input data copied to . This
    should pr passed to the function under test as target data to be
    transformed. example: [TMP_DIR]/[TEST]/[CASE]/input

    actual(str): absolute path to the transformation output data. If this is
    the same as expected, a test would pass. example:
    [TMP]]/[TEST]/[CASE]/golden



    """

    def __init__(
        self, test_data, test_name, test_case, temp_dir, update_golden=False
    ):
        """init data. see class docstring for more details

        :param str test_data: project directory containing all test
        data. This is prepended to all the relevatn test case resource
        paths

        :param str test_name: test name from request.node.originalname

        :param str test_case: test case from parametrize

        :param str temp_dir: temporary directory for the test case

        if update_golden is  True
        """
        self.update_golden = update_golden
        self.test_name = test_name
        self.test_case = test_case
        self.golden = os.path.join(test_data, test_name, test_case, "golden")
        self.expected = os.path.join(
            temp_dir, test_name, test_case, "expected"
        )
        self.input = os.path.join(test_data, test_name, test_case, "input")
        self.actual = os.path.join(temp_dir, test_name, test_case, "actual")
        self.temp_case_dir = os.path.join(temp_dir, test_name, test_case)
        self.project_case_dir = os.path.join(test_data, test_name, test_case)
        # create the case path. populate will create the content subdirs
        mkdir_p(self.temp_case_dir)

        # if update_golden is  true, DO NOT populate temp until AFTER we use
        # the function under test to generate new golden contents
        if update_golden:
            self._delete_golden()
        else:
            self._populate_temp()

    def _populate_temp(self):
        """Copy test data to temp"""
        from distutils.dir_util import copy_tree

        if not os.path.isdir(self.golden):
            msg = (
                "Golden is missing: {}. Are you running with "
                "update_golden=True, but forgetting to create "
                "the new golden?".format(self.golden)
            )
            raise RuntimeError(msg)
        copy_tree(self.input, os.path.join(self.temp_case_dir, "input"))
        copy_tree(self.golden, self.expected)

    def _compare_use_filecmp(self):
        """use filecmp to compare actual and expected
        Return a result tuple":

        example


        :param str output_string: log string to process

        :rtype: bool, Tuple[str, str, str]
        """
        import filecmp
        from treecrawl.utility import compare_directories

        if os.path.isfile(self.input):
            res = filecmp.cmp(self.actual, self.expected, shallow=False)
            res = res, (self.input, self.actual, self.expected)
        elif os.path.isdir(self.input):
            succeeded = compare_directories(self.actual, self.expected)
            res = succeeded, (self.input, self.actual, self.expected)
        else:
            raise RuntimeError("Expected path to file or directory.")
        return res

    def _delete_golden(self):
        """



        :param str output_string: log string to process

        :rtype: List[Dict[str, str]]
        """
        from shutil import rmtree

        try:
            rmtree(self.golden)
        except FileNotFoundError:
            # don't care if it doesn;t exist
            pass

    def compare(self):
        """Run all comparisons

        If self.update_golden is True, golden needs to be updated externally
        by running the function under test and pointing its output to the
        golden path.  compare deals with this by conditionally run

        :param str output_string: log string to process

        :rtype: List[Dict[str, str]]
        """
        if self.update_golden:
            self._populate_temp()
        results = []
        results.append(self._compare_use_filecmp())
        return results



import logging
import os
from .utility import string_to_log_level, validate_path
from treecrawl.utility import create_module_logger

module_name = str(__name__)
module_logger = create_module_logger(module_name)


class Transformer(object):
    """Transform a file or directory

    input and output should both be paths to files OR both be directories
    with the same structure

    """

    dry_run_prefix = "SKIPPING! (DRY RUN): "

    def __init__(
        self, input=None, output=None, log_level="INFO", dry_run=True
    ):
        import json

        if input is None:
            self._input = os.getcwd()
        else:
            self._input = validate_path(input)
        if output is None:
            self._output = self._input
        else:
            self.output = output
        self.logger = logging.getLogger(
            module_name + "." + self.__class__.__name__
        )
        self.log_level = string_to_log_level(log_level)
        self.logger.setLevel(self.log_level)
        self.dry_run = dry_run

        msg_dict = {
            "input": self.input,
            "output": self.output,
            "log_level": self.log_level,
            "dry_run": str(self.dry_run),
        }
        self.logger.info(json.dumps(msg_dict))

    def add_dry_run_prefix(self, mm):
        """if the dry_run flag is set prepend the message with skipping..

        :param str mm: message to prepend

        :rtype: str
        """
        if self.dry_run:
            return Transformer.dry_run_prefix + mm
        return mm

    @property
    def input(self):
        return self._input

    @input.setter
    def input(self, value):
        self._input = validate_path(value)
        return self._input

    @property
    def output(self):
        return self._output

    @output.setter
    def output(self, value):
        self._output = value
        return self._output

    def in_place(self):
        """Return true if teh output will overwrite the input

        :rtype: bool
        """
        return self._input == self._output

    def source_dest_as_dict(self):
        """If the target us a directory, return dict of input:output files

        The logic for selecting targets can be customized by overriding this
        method.

        :rtype: Dict[str, str]
        """
        from treecrawl.utility import (
            get_all_files,
            output_file_from_input_file,
        )

        res = {}
        if os.path.isfile(self.input):
            return {self.input: self.output}

        input_files = get_all_files(self.input)
        for file in input_files:
            if self.is_target(file):
                # transform input file and write to destination
                # in the same relative path in the output dir
                res[file] = output_file_from_input_file(
                    self.input, self.output, file
                )

        return res

    def is_target(self, i_file):
        """Return True is the file meets criteria to be transformed

        This is used by filter_files to build a target list.  The base
        implementation edits all files. Override this method to customizing
        file targeting

        WARNING!! I use opt-in targeting because treecrawl functions do not
        protect your binary files from being manipulated like  text files

        :param str i_file: abs path to target candidate

        :rtype: bool
        """
        raise NotImplementedError

    def transform(self, source_file, destination_file):
        """Override this with transformation logic

        CRITICAL: transform must check the value of self.dry_run and do the
        right thing.  use Transformer.add_dry_run_prefix to make the
        conditional logging a little easier

        read the source_file, do whatever and write to destination_file

        writing files may need to be encoding aware. look at methods that help
        with that like write_string_to_output or write your own

        :param str source_file: read this file as input
        :param str destination_file: write transformed file here

        """
        raise NotImplementedError

    def run(self):
        for k, v in self.source_dest_as_dict().items():
            if v is None:
                v = k
            self.transform(k, v)

    @staticmethod
    def write_string_to_output(s, o):
        """writes a string to a an absolute file path

        also creates necessary directories along the way

        :param str output_string: log string to process

        :rtype: List[Dict[str, str]]
        """
        from treecrawl.utility import string_to_file, mkdir_p

        if type(s) != str:
            msg = "Expected string input. Got {}".format(str(type(s)))
            raise RuntimeError(msg)
        # ensure directory pah exists
        mkdir_p(o, is_file=True)
        string_to_file(s, o)



#!/usr/bin/env python

"""Tests for `treecrawl` package."""
import os
import pytest
from treecrawl.utility import (
    find_path_to_subdirectory,
    mkdir_p,
    output_file_from_input_file,
    get_all_files,
    validate_path,
)
from treecrawl.casehelper import CaseHelper


def test_find_path_to_subdirectory(tmp_path):
    import os

    target_dir = os.path.join(
        str(tmp_path), *["some", "path", "to", "testdata"]
    )
    mkdir_p(target_dir)
    ff = find_path_to_subdirectory("testdata", search_path=str(tmp_path))
    assert ff[0].endswith("testdata")


@pytest.mark.parametrize(
    "test_case,file_count",
    [("happy_path", 3)],
)
def test_get_all_files(
    test_case, file_count, tmp_path, request, testdata, update_golden
):
    """This is a stupid redundant test but wtf

    c CaseHelper copies golden to tmp/expected and input to tmp/actual.
    file-count should match the total number of files that exist in tmp
    after this happens.

    """

    # Instantiate a TestData object with the name of the test (originalname)
    c = CaseHelper(
        testdata,
        request.node.originalname,
        test_case,
        str(tmp_path),
        update_golden=update_golden,
    )
    # Execute against initial data
    res = get_all_files(c.temp_case_dir)
    assert len(res) == file_count


def test_rel_pos_validate(tmp_path):
    d = tmp_path / "rel_exists"
    d.mkdir()

    orig_wd = os.getcwd()
    os.chdir(str(tmp_path))
    assert validate_path("rel_exists") == str(d)
    os.chdir(orig_wd)


def test_rel_neg_validate(tmp_path):
    d = tmp_path / "rel_exists"

    orig_wd = os.getcwd()
    os.chdir(str(tmp_path))
    with pytest.raises(RuntimeError, match="Invalid path: {}".format(str(d))):
        validate_path("rel_exists")
    os.chdir(orig_wd)


def test_none_pos_validate(tmp_path):
    orig_wd = os.getcwd()
    os.chdir(str(tmp_path))
    assert validate_path() == str(tmp_path)
    os.chdir(orig_wd)


def test_abs_pos_validate(tmp_path):
    d = tmp_path / "abs_exists"
    d.mkdir()
    assert validate_path(str(d)) == str(d)


def test_abs_neg_validate(tmp_path):
    d = tmp_path / "abs_exists"
    with pytest.raises(RuntimeError, match="Invalid path: {}".format(str(d))):
        validate_path(str(d))


# this was an interesting experiment using python magic
# to inspect filkes a litt=le closer. I've abandoned it because
# there's not enough value in the approach to account for adding
# an external dependency
@pytest.mark.skip
@pytest.mark.parametrize(
    "test_case,type",
    [
        ("a_directory", "dir"),
        ("ascii.txt", "ASCII text"),
        ("min_yaml.yml", "ASCII text"),
        ("utf8_unlabeled.txt", "UTF-8 Unicode text"),
        ("yaml.yml", "ASCII text"),
    ],
)
def test_path_type(test_case, type, request, testdata):
    """Run and compare results to expected"""
    # target = os.path.join(testdata, request.node.originalname, test_case)
    assert True
    # assert path_type(target) == type


def test_output_file_from_input_file():

    res = output_file_from_input_file(
        "/a/b/c/", "/d/f/g", "/a/b/c/x/y/z/file.txt"
    )
    assert res == "/d/f/g/x/y/z/file.txt"


def test_locate_subdir():
    from treecrawl.utility import locate_subdir

    res = locate_subdir("testdata")
    assert os.path.isdir(res)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

# abandoned utility.path_type experiment
requirements = [
    # "python-magic==0.4.18"
]

setup_requirements = []

dev_requirements = {
    "dev": [
        "pip==20.2.2",
        "setuptools==49.2.0",
        "black==19.10b0",
        "bump2version",
        "docutils==0.16",
        "flake8==3.8.3",
        "nox==2020.5.24",
        "pytest==5.4.3",
        "Sphinx==3.1.2",
        "twine==3.2.0",
        "wheel==0.34.2",
        "check-manifest==0.42",
    ]
}

test_requirements = [
    "pytest>=3",
]

setup(
    author="Nate Marks",
    author_email="npmarks@gmail.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="libraries to make it easier to "
    "maniuplate files in a directory tree",
    install_requires=requirements,
    extras_require=dev_requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="treecrawl",
    name="treecrawl",
    packages=find_packages(include=["treecrawl", "treecrawl.*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/natemarks/treecrawl",
    version="0.1.23",
    zip_safe=False,
)



"""Top-level package for treecrawl."""

__author__ = """Nate Marks"""
__email__ = "npmarks@gmail.com"
__version__ = "0.1.23"


from .transformer import Transformer
from .casehelper import CaseHelper
from .utility import (
    create_module_logger,
    compare_directories,
    file_to_string,
    file_name_from_path,
    output_file_from_input_file,
    locate_subdir,
    find_path_to_ancestor,
    find_path_to_subdirectory,
    mkdir_p,
    string_to_file,
    string_to_log_level,
    get_all_files,
    strip_prefix,
    strip_suffix,
    validate_path,
)

__all__ = (
    "Transformer",
    "CaseHelper",
    "create_module_logger",
    "compare_directories",
    "file_to_string",
    "find_path_to_ancestor",
    "find_path_to_subdirectory",
    "mkdir_p",
    "string_to_file",
    "string_to_log_level",
    "get_all_files",
    "strip_prefix",
    "strip_suffix",
    "validate_path",
    "locate_subdir",
    "output_file_from_input_file",
    "file_name_from_path",
)
